---
short-description: "Apertis for platform developers"

authors:
    - name: Emanuele Aina, Sjoerd Simons
      email: emanuele.aina@collabora.com, sjoerd.simons@collabora.co.uk
      years: [2017]

license: CC-BY-SAv4.0
...

# Apertis for platform developers

Apertis is a versatile open source infrastructure tailored to the automotive
needs and fit for a wide variety of electronic devices. Security and
modularity are two of its primary strengths.  Apertis provides a feature-rich
framework for add-on software and resilient upgrade capabilities.
Beyond an operating system, it offers new APIs, tools and cloud services.

Apertis is not just a Ubuntu/Debian-derived Linux distribution:
it comprises code hosting, code review tools, package build and image
generation services, and an automated testing infrastructure with the aim of
providing a clean, reliable environment to let developers go from sources to
deployable system images in the most dependable way.  It aims to be the
integration point for different product lines with different goals and
different schedules, but all sharing the same common core.

The set of packages shipped in Apertis provides a baseline that can be picked
and chosen to quickly create deployable images for different products.

The goal of Apertis is to maximize the sharing across products, with the aim to
improve time to market and reduce the efforts on long-term maintenance, in
particular to enable quick and consistent response times for  security issues
in internet-enabled products.

## From sources to deployment

Apertis is built on top of Ubuntu/Debian `deb` packages for their high quality
and modularity.  All the source packages are stored in the [Open Build Service
(OBS) instance](https://build.collabora.com) provided by Collabora where the
Apertis OBS projects are hosted: each release is currently composed of a set of
stacked projects:

* `target`: packages intended for use in product images
* `development`: additional packages needed to build `target` and development tools
* `hmi`: packages for the current reference HMI on top of `target`
* `sdk`: packages to build the SDK virtual machine recommended for development

In case of common interests new packages can easily be introduce in the relevant
OBS projects. It's also possible to create additional projects in OBS for e.g.
product specific software packages or product specific modifications that build
on top of the common baselines but aren't suitable for more general inclusion.

OBS takes care of automatically building the package sources for all the
configured architectures (`x86_64`, `armv7l`, `aarch64`) and publishes the
results as signed APT repositories. Each package is built in a closed,
well-defined environment where OBS automatically installs all the build tools
and resolves dependencies from a clean state using the packages it has built
already: if any dependency is missing the build fails and the developer is
notified.

Most of the package sources get automatically updated from the latest Ubuntu
LTS release or have been manually picked from later Ubuntu/Debian releases when
more up-to-date versions have been deemed beneficial (one example of that is
the `systemd` package). This allows Apertis to share bugfixes and security
fixes with the efforts done by the wider Ubuntu and Debian communities.

Packages specific to Apertis have their sources stored in [git
repositories hosted on the Apertis infrastructure](https://git.apertis.org).
The git infrastructure to manage the repositories is very basic at the moment,
however deploying a Apertis [GitLab](https://gitlab.com/) instances is planned
to allow apertis users and contributors to more easily share and host code.

For packages hosted on the Apertis git infrastructure the [Apertis Jenkins instance](https://Jenkins.apertis.org/) 
is configured to [automatically fetch tagged commits](https://wiki.apertis.org/Guidelines/HowToReleasePackages#Release_process)
and push them to OBS after initial smoketesting to get a continuous and
hands-off update flow.

After OBS has finished building a package, the results get published in a
Debian package repository. The open-source packages from Apertis can be found
in the public [Apertis Debian repositories](https://repositories.apertis.org/apertis/).

The packages in these repositories can then be used to build images suitable
for deployment onto a variety of targets, e.g. hardware boards (reference
or product specific), virtual machines (e.g. for development), container images etc etc.

The overall strategy for building these deployments is to separate it in
various stages starting with early common stages (e.g. a common rootfs) and
then further specializing in hardware or deployment specific additions (e.g.
kernel and bootloader for a specific board). The result of each stage is
typically referred to as a pack, typically we have at least two packs:
* `ospack`: the set of user space binary packages that are not specific to a particular SoC/platform or deployment method
* `hwpack`: contains the hardware-specific packages for a particular SoC/platform, like bootloader, kernel, codecs, GL stack etc

The reason for this split is that it allows the creation of just one SoC,
platform or even board specific hardware pack which can be combined with a
selection of ospacks. Typically Apertis has a `target` ospack with only the
software meant to go into the final product and a `development` ospack which
in addition contains extra developer tooling.

For instance the `target` and `development` ospacks for `armhf` could be
combined with the i.MX6 Sabrelite and Raspberry Pi 2 hwpacks to generate four
possible combinations of flashable images.

Generating images does not involve rebuilding all the packages from source and
can thus be done quite quickly and flexibly.

Both ospack/hwpack creation and image generation are controlled through YAML
files, configuring partitions, bootloaders, which packages gets installed,
which overlays is to be applied, and arbitrary customization shell scripts to
be run over the rootfs in a QEMU-based virtualized environment.

This process is usually automatically run by Jenkins jobs, but during
development can be run on developers machines as well, fetching packages from
the same OBS binary repositories.

Once images are generated, other Jenkins jobs will schedule a batch of tests on
the [LAVA instance hosted by Collabora](https://lava.collabora.co.uk) which
will take care of deploying the freshly generated images on actual target
devices running in the Collabora device farm, and of controlling them over
serial connections to run the defined testcases and gather the results.

![](media/apertis-functional-view.png)

## The workflow

The keypoints in the workflow for Apertis components are thus:

* the [VirtualBox-based Apertis SDK](https://wiki.apertis.org/VirtualBox) is
* used for development sources are stored on the [git code
* hosting](https://git.apertis.org) with `deb` control files
* [Phabricator](https://phabricator.apertis.com) is used for project management
* and code review [Jenkins](https://jenkins.apertis.org) pushes tagged commits
* to OBS [OBS](https://build.collabora.co.uk) builds sources packages and
* generates binary packages in controlled environments every night Jenkins jobs
* generate ospacks and hwpacks from the repositories built by OBS the generated
* ospacks and hwpacks are combined by Jenkins jobs to produce deployable images
* on success, Jenkins jobs trigger on-device tests on
* [LAVA](https://lava.collabora.co.uk) to check the produced images other
* Jenkins jobs check if pacakges included in the images are tagged with task
* identifiers and closes them in Phabricator

### Software components packaging (deb)

For all the software components meant to be included in the images Apertis uses
the [`deb` packaging format](https://wiki.debian.org/Packaging) used by Debian
and Ubuntu.

To package a component from scratch, Debian provides [a short guide to get
started](https://wiki.debian.org/Packaging/Intro).

The VirtualBox-based [Apertis SDK virtual machine images](https://wiki.apertis.org/VirtualBox) ship with all the needed tools
installed, providing a reliable, self-contained environment ready to be used.

Once the component has been packaged, its sources can be uploaded to OBS in a
user-specific project, such that the developer is free to experiment and
iterate until the component is ready to be submitted to the appropriate OBS
project.

### Open Build Service (OBS)

Open Build Service is the backbone of the package building infrastructure in
Apertis.  It stores source packages and builds them in controlled environments
for all the configured CPU architectures.

The source packages are grouped in projects that can be stacked such that each
stacked project will automatically share the packages in the underlying
projects with all the other projects stacked on top of them.

This provides a lot of flexibility to handle different groups working on
different products with different schedules while still sharing a common core.

OBS also enables developers to easily [branch and customize existing packages](https://wiki.apertis.org/Building_a_Debian_package) and build them in
isolated, user-specific sandboxes. Developers are free to experiment and when
the updates are ready they can be merged back into the original project or
moved to a new stacked project.

### Code hosting and review process

Apertis heavily [relies on Phabricator](https://phabricator.apertis.org)
for [the development of Apertis-specific components](https://wiki.apertis.org/Guidelines/Contribution_process#Contribution_process_2).

Those have their [source code hosted on Apertis servers](https://git.apertis.org/)
which is automatically submitted to OBS when release tags are pushed.

Each patch is supposed to be [reviewed on Phabricator](https://wiki.apertis.org/Guidelines/Contribution_process#Contribution_process_2)
before it can land in the git repository and thus on OBS.

In the near future Apertis plans to use a dedicated GitLab instance to let
developers self-manage their git repositories.

### Jenkins automation

All the automated tasks in Apertis are orchestrated
by the [Apertis Jenkins instance](https://jenkins.apertis.org/):

* build-testing patches attached to Phabricator
* submitting tagged commits to OBS
* running ospack and hwpack creation jobs
* generating the deployable images
* triggering tests on the devices attached to LAVA
* autoclosing Phabricator tasks when marked packages are included in a succesful image build

### Image creation

Ospacks, hwpacks and how they should be combined to generated images are
defined through YAML files.

This is an example configuration for an ARMv7 image, `image-armhf.yaml`:

{{ ../examples/image-armhf.yaml }}

And this is the `ospack-armhf.yaml` configuration for the ARMv7 ospack:

{{ ../examples/ospack-armhf.yaml }}

Collections of images are built every night such that developers can always
dowload the latest image to deploy it to a target device and start using it
immediately.

Images are then published on the [deployable image hosting website](https://images.apertis.org)

### Automated testing with LAVA

To ensure the continued quality of the generated images, a set of automated
on-device tests is run for every image so issues can be found early if they arise
and handled in a timely fashion by developers.

## Key technologies

Apertis makes heavy use of some technologies for its purposes:

* Ubuntu/Debian packages
* Systemd for application lifecycle tracking
* AppArmor for policy enforcement
* OSTree/Flatpak for safe and efficient deployments
* DBus for privilege separation
* Wayland for graphics
* GStreamer for multimedia playback

### OTA update strategies

Apertis currently uses Btrfs snapshot for OTA updates but is now moving towards
a full system update strategy based on OSTree which has some benefits over the
Btrfs-based solution:

* works in containers
* works on flash-specific filesystems like UBIfs
* smaller downloads

### Application framework

Apertis ships an application framework
that provides flexibility and modularity post-deployment:

* deploying applications can be done independently from full system updates
* network access, inter-process communications and file access policies are enforced through AppArmor
* failed application deployments are safely rolled back
* works both on targets, virtual machines and containers
* Flatpak-based application bundle file format
* the `ade` command-line tool simplifies the generation of application bundles

### Domain separation

Apertis is designed to work in setups where tasks are split over different
domains, which can be connected SoCs, virtual machines or containers, for
instance on setups where a privileged domain has no direct Internet access but
relies on a separated, more constrained domain to access network services and
validate any communication.
